
############# DISCLAIMER (Please do not edit) ############################

# The present routine as been assembled by JF Doussin (March 2017)
# in the framework of the Eurochamp-2020 Infrastructure activity funded 
# by European Union's Horizon 2020 research and innovation programme 
# under grant agreement No 730997".

# From a txt file easily produced from excel, it aims 
# 1/ at formating all the parameter given in excel in column into the EDF format 
# 2/ at plotting the data in a series of graphs one above the other in a pdf file

# The data must be given as in indicated in the model file "exemple.txt". 
# !! CAREFUL !!! the first column must contain data/hour data in the format dd/mm/yyyy hh:mm:ss	
# (by default seconds are missing in the usual Excel time format) 
# and the column have to be separated with a tab

# User inputs are requested at lines 29 to 42 and from 65 to 80.

# Versioning 
# v1.0 => v1.1 :  a few bugs related with PDF/PNG output saving were fixed (in particular, the call
# for new output files was set outside - before - the looprequested to plot several graphs on a single page)



library(png)
library('png')
library('grid')
library(ggplot2)
library(scales)

############################## empty stack ################################ 
rm(list = ls(all = TRUE))

# !!!!!!!!!!!!! WARNING ! USER INPUT REQUESTED !!!!!!!!!!!!!!!!!!!!
# ------ Indicate  where the script and logos are stored ------------------
scriptdir<-"C:/Users/Doussin JF/Documents/JFD/R/Eurochamp-2020/Excel to EDF/"

# !!!!!!!!!!!!! WARNING ! USER INPUT REQUESTED !!!!!!!!!!!!!!!!!!!!
# ------ Indicate  where the data are stored ------------------
indir<-"C:/Users/Doussin JF/Documents/JFD/R/Eurochamp-2020/Excel to EDF/"

######################### Read all the lines ############################# 

# !!!!!!!!!!!!! WARNING ! USER INPUT REQUESTED !!!!!!!!!!!!!!!!!!!!
# ------ Indicate  the name of the file --------------------------------
infile<-paste(indir,"exemple.txt",sep="")

###########################################################################

#Read all the lines
con <- file(infile, open = "r")
lines <- readLines(con)
close(con)

################ Prepare the file in a standard way ######################## 

# change possible comma in dot for the right decimal writting
write.table(gsub(',','.',lines),append=F,quote=F,sep=" ",file='tmp',col.names=F,row.names=F)

#################### create output EDF and PDF files ########################

outfile_edf<-paste(substr(infile,1,(nchar(infile)-4)), ".edf", sep="")
outfile_png<-paste(substr(infile,1,(nchar(infile)-4)), ".png", sep="")
outfile_pdf<-paste(substr(infile,1,(nchar(infile)-4)), ".pdf", sep="")
 
# !!!!!!!!!!!!! WARNING ! USER INPUT REQUESTED !!!!!!!!!!!!!!!!!!!!
# ----- Metadata manual entry (update the following lines accordingly)------
fileConn<-file(outfile_edf, open="wt")
writeLines("X_HEADER=NETCDF_GLOBAL", fileConn)
writeLines("PI_NAME=Doussin Jean-Francois", fileConn)
writeLines("WORKGROUP=MEREIA", fileConn)
writeLines("INSTITUTE=LISA", fileConn)
writeLines("EMAIL=jean-francois.doussin@lisa.u-pec.fr", fileConn)
writeLines("ADDRESS=61, avenue du General de Gaulle 94010 CRETEIL Cedex", fileConn)
writeLines("", fileConn)
writeLines("TITLE=Test measurement", fileConn)
writeLines("DATA_CATEGORY=EXPERIMENT", fileConn)
writeLines("EXPERIMENT=Exemple of photolysis experiment", fileConn)
writeLines("CAMPAIGN=Exemple test", fileConn)
writeLines("TYPE_OF_DATA=MEASUREMENT", fileConn)
writeLines("STATUS_OF_FILE=FINAL", fileConn)
writeLines("VERSION=1.0", fileConn)
writeLines("PLATFORM=CHAMBER", fileConn)
writeLines("NAME_OF_PLATFORM=CESAM", fileConn)
writeLines("DESCRIPTION=http://cesam.cnrs.fr/", fileConn)
writeLines("", fileConn)

writeLines("X_HEADER=NETCDF_TIME", fileConn)
writeLines("SHORT_NAME(0)=time", fileConn)
writeLines("LONG_NAME(0)=time", fileConn)


####################### Count the number of data column ############################

data<-read.table('tmp',sep="\t", skip=0)
nbcol<-ncol(data) 

######################## Read the numeric data ########################
titres<-lines[2]
titres <- gsub(' ','',titres)
titres<- gsub("SHORT_NAME","TIME",titres)
nomcol=unlist(strsplit(titres, split="\t"))

numdata<-read.table('tmp',sep="\t", skip=7,col.names = nomcol)

######################## Define time ######################## 
time1<-numdata[,"TIME"]
Time_exp <- as.POSIXct(strptime(time1, "%d/%m/%Y %H:%M:%S"),tz="UTC")
date_exp<- titres<-lines[8]
date_exp<- substr(date_exp, 1, 10)

Time_start <- as.POSIXct(strptime(paste(date_exp,"00:00:00"), "%d/%m/%Y %H:%M:%S"),tz="UTC")
Start_Time <-as.numeric(Time_start)
Time_sec<-as.numeric(Time_exp)

Time_sec<-Time_sec-Start_Time

############################ Metadata automatic entry  #############################
lines_edf <- paste("UNITS(0)=seconds since",Time_start, "00:00:00 UTC")
writeLines(lines_edf, fileConn)

writeLines("", fileConn)
writeLines("X_HEADER=NETCDF_PARAMETER", fileConn)

for (icol in seq(2,nbcol)){

  lines_edf <- paste("LONG_NAME(",icol-1,")=",data[1,icol], sep="")
  writeLines(lines_edf, fileConn)
    lines_edf <- paste("SHORT_NAME(",icol-1,")=",data[2,icol], sep="")
  writeLines(lines_edf, fileConn)  
  lines_edf <- paste("UNITS(",icol-1,")=",data[3,icol], sep="")
  writeLines(lines_edf, fileConn)  
  lines_edf <- paste("DESCRIPTION(",icol-1,")=",data[4,icol], sep="")
  writeLines(lines_edf, fileConn)  
  lines_edf <- paste("ACCURACY_",data[5,icol],"(",icol-1,")=",data[6,icol], sep="")
  writeLines(lines_edf, fileConn)
  lines_edf <- paste("INSTRUMENT(",icol-1,")=",data[7,icol], sep="")
  writeLines(lines_edf, fileConn) 
  writeLines("", fileConn)
}

writeLines("X_HEADER=ENZ", fileConn)
lines_edf <- paste("COLUMN 1=TIME", sep="")
writeLines(lines_edf, fileConn)

for (icol in seq(2,nbcol)){  
  lines_edf <- paste("COLUMN ",icol,"=",data[2,icol], sep="")
  writeLines(lines_edf, fileConn)  
}

lines_edf <- paste("NUMBER OF COLUMNS=", nbcol, sep="")
writeLines(lines_edf, fileConn)
lines_edf <- "&&&&&&&&&&&&&&&&&&&&&&&&&"
writeLines(lines_edf, fileConn)
close(fileConn)

dataEDF <- data.frame(Time_sec,numdata[,2:nbcol])

write.table(dataEDF, file = outfile_edf, append = TRUE, quote=TRUE, sep="\t",
            na = "-9999", dec = ".", row.names = FALSE, col.names = FALSE)


##############################
#     Plot EDF data          #
##############################

#lecture de toutes les lignes
con <- file(outfile_edf, open = "r")
lines <- readLines(con)
close(con)
###############################   

################ Prepare the file in a standard way ################ 
lines <- gsub('-9999','0',lines)
write.table(gsub(',','.',lines),append=F,quote=F,sep=" ",file='tmp',col.names=F,row.names=F)


#Recherche de "NUMBER OF COLUMNS="
nbcol<-lines[which(substr(lines,1,17)=="NUMBER OF COLUMNS")]
#recupration du nombre de colonne
nbcol<-as.integer(unlist(strsplit(nbcol,"="))[2])


Title1<-lines[which(substr(lines,1,8)=="CAMPAIGN")]
Title1<-gsub("CAMPAIGN=","",Title1)

Title2<-lines[which(substr(lines,1,10)=="EXPERIMENT")]
Title2<-gsub("EXPERIMENT=","",Title2)

MainTitle<-paste(Title1, "\n", Title2,"\n", date_exp,sep=" ")


#cherche le UNITS(0)
unit<-lines[which(substr(lines,1,8)=="UNITS(0)")]
#recupration date et heure  T0
date_ini<-unlist((strsplit(unit," ")))[3]
heure_ini<-unlist((strsplit(unit," ")))[4]

#lit les donnes  partir de lines "&"
data<-read.table('tmp',skip=which(substr(lines,1,1)=="&"))


#dfinition du temps
date_x<-as.POSIXct(data[,1],paste(date_ini,heure_ini,sep=" "),tz="UTC")
date_label <- gsub('/','-',date_ini)

###################### Produce Report PNG #############################
outfile4<-outfile_png
png(outfile4, width = 2100, height = 2970) # Open a new png file
img1 <- readPNG(paste(scriptdir,"eurochamp2020_logo.png",sep="")) 
Logo_E2020 <- rasterGrob(img1, x=unit(0.01, "npc"),y=unit(0.999, "npc"), width = 0.12, height = 0.06, just=c("left", "top")) 


for (icol in seq(2,nbcol)){
  #rcupration de SHORT_NAME
  if (icol > 10) {
    name<-lines[which(substr(lines,1,14)==paste("SHORT_NAME(",(icol-1),")",sep=""))]
  } else {
    name<-lines[which(substr(lines,1,13)==paste("SHORT_NAME(",(icol-1),")",sep=""))]
  }
  #recuperation de name
  name<-unlist((strsplit(name,"=")))[2]
  
  
  #rcupration de UNITS
  if (icol > 10) {
    units<-lines[which(substr(lines,1,9)==paste("UNITS(",(icol-1),")",sep=""))]
  } else {
    units<-lines[which(substr(lines,1,8)==paste("UNITS(",(icol-1),")",sep=""))]
  }
  
  #recupration de l'unit
  units<-unlist((strsplit(units,"=")))[2]
  
  
  ####### Define function for beautiful labels #############
  
  fmt_dcimals <- function(decimals=0){
    function(x) format(x,digits = decimals,scientific = TRUE)
  }
  
  fancy_scientific <- function(l) {
    # turn in to character string in scientific notation
    l <- format(l, scientific = TRUE, width=9,digits=2)
    l <- gsub("0e\\+00","0",l)
    # quote the part before the exponent to keep all the digits
    l <- gsub("^(.*)e", "'\\1'e", l)
    # turn the 'e+' into plotmath format
    l <- gsub("e", "%*%10^", l)
    # return this as an expression
    parse(text=l)
  }
  ############# Plot the data - one plot over the other ####################
  
  # --------------------- Insert title of the experiment --------------------
  
  PlotTitle<-name
  
   # ------------------ setting time limits of the plot -------------------
  x_min<-min(date_x)
  x_max<-max(date_x)
  lims <- as.POSIXct(strptime(c(x_min , x_max), format = "%Y-%m-%d %H:%M"))    
  
  # ---------------------- Make the plot -------------------------------
  p1<-ggplot(data=data, aes(x=date_x, y=data[,icol])) + 
    geom_line() +
    xlab("Time") +
    ylab(paste(name,"\n(",units,")",sep="")) +
    labs(fill = bquote('Maximum number')) +   
    scale_fill_gradient(low="white", high="white") + 
    scale_x_datetime(limits = lims, expand = c(0,0),breaks = date_breaks("1 hour"),
                     labels = date_format("%H:%M")) +
    scale_y_continuous(labels=fancy_scientific) +
    theme(legend.title = element_text(colour="white", size=8), 
          legend.text = element_text(colour="white", size=-0.5*nbcol+15, face="bold"),
          axis.text.x = element_text(colour="grey20",size=-0.5*nbcol+15,angle=0,hjust=.5,vjust=.5,face="plain"),
          axis.text.y = element_text(colour="grey20",size=-0.5*nbcol+15,angle=0,hjust=1,vjust=0,face="plain"),  
          axis.title.x = element_text(colour="grey20",size=-0.5*nbcol+15,angle=0,hjust=.5,vjust=0,face="plain"),
          axis.title.y = element_text(colour="grey20",size=-0.5*nbcol+14,angle=90,hjust=.5,vjust=.5,face="plain"))
  
  if (nbcol > 2) {
    pushViewport(viewport(layout = grid.layout(nrow=nbcol, ncol=3,widths = unit(c(1,4,1), "null"))))
    print(grid.draw(Logo_E2020), vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
    print(p1, vp = viewport(layout.pos.row = icol, layout.pos.col = 2)) 
    grid.text(MainTitle, vp = viewport(layout.pos.row = 1, layout.pos.col = 2))
    
  } else {
    pushViewport(viewport(layout = grid.layout(nrow=2, Heights = unit(c(1,4), "null"), ncol=3,widths = unit(c(1,4,1), "null"), heigths = unit(c(.1,1),"null"))))
    print(grid.draw(Logo_E2020), vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
    print(p1, vp = viewport(layout.pos.row = icol, layout.pos.col = 2)) 
    grid.text(MainTitle, vp = viewport(layout.pos.row = 1, layout.pos.col = 2))
  }
}


dev.off() # Close the file

###################### Produce Report PDF #############################
outfile5<-outfile_pdf
pdf(outfile5, width = 21/2.5, height = 29.70/2.5) # Open a new pdf file
img1 <- readPNG(paste(scriptdir,"eurochamp2020_logo.png",sep="")) 
Logo_E2020 <- rasterGrob(img1, x=unit(0.01, "npc"),y=unit(0.999, "npc"), width = 0.12, height = 0.06, just=c("left", "top")) 


for (icol in seq(2,nbcol)){
  #rcupration de SHORT_NAME
  if (icol > 10) {
    name<-lines[which(substr(lines,1,14)==paste("SHORT_NAME(",(icol-1),")",sep=""))]
  } else {
    name<-lines[which(substr(lines,1,13)==paste("SHORT_NAME(",(icol-1),")",sep=""))]
  }
  #recuperation de name
  name<-unlist((strsplit(name,"=")))[2]
  
  
  #rcupration de UNITS
  if (icol > 10) {
    units<-lines[which(substr(lines,1,9)==paste("UNITS(",(icol-1),")",sep=""))]
  } else {
    units<-lines[which(substr(lines,1,8)==paste("UNITS(",(icol-1),")",sep=""))]
  }
  
  #recupration de l'unit
  units<-unlist((strsplit(units,"=")))[2]
  
  
  ####### Define function for beautiful labels #############
  
  fmt_dcimals <- function(decimals=0){
    function(x) format(x,digits = decimals,scientific = TRUE)
  }
  
  fancy_scientific <- function(l) {
    # turn in to character string in scientific notation
    l <- format(l, scientific = TRUE, width=9,digits=2)
    l <- gsub("0e\\+00","0",l)
    # quote the part before the exponent to keep all the digits
    l <- gsub("^(.*)e", "'\\1'e", l)
    # turn the 'e+' into plotmath format
    l <- gsub("e", "%*%10^", l)
    # return this as an expression
    parse(text=l)
  }
  ############# Plot the data - one plot over the other ####################
  
  # --------------------- Insert title of the experiment --------------------
  
  PlotTitle<-name
  
 
  # ------------------ setting time limits of the plot -------------------
  x_min<-min(date_x)
  x_max<-max(date_x)
  lims <- as.POSIXct(strptime(c(x_min , x_max), format = "%Y-%m-%d %H:%M"))    
  
  # ---------------------- Make the plot -------------------------------
  p1<-ggplot(data=data, aes(x=date_x, y=data[,icol])) + 
    geom_line() +
    xlab("Time") +
    ylab(paste(name,"\n(",units,")",sep="")) +
    labs(fill = bquote('Maximum number')) +   
    scale_fill_gradient(low="white", high="white") + 
    scale_x_datetime(limits = lims, expand = c(0,0),breaks = date_breaks("1 hour"),
                     labels = date_format("%H:%M")) +
    scale_y_continuous(labels=fancy_scientific) +
    theme(legend.title = element_text(colour="white", size=8), 
          legend.text = element_text(colour="white", size=-0.5*nbcol+15, face="bold"),
          axis.text.x = element_text(colour="grey20",size=-0.5*nbcol+15,angle=0,hjust=.5,vjust=.5,face="plain"),
          axis.text.y = element_text(colour="grey20",size=-0.5*nbcol+15,angle=0,hjust=1,vjust=0,face="plain"),  
          axis.title.x = element_text(colour="grey20",size=-0.5*nbcol+15,angle=0,hjust=.5,vjust=0,face="plain"),
          axis.title.y = element_text(colour="grey20",size=-0.5*nbcol+14,angle=90,hjust=.5,vjust=.5,face="plain"))
  
  if (nbcol > 2) {
    pushViewport(viewport(layout = grid.layout(nrow=nbcol, ncol=3,widths = unit(c(1,4,1), "null"))))
    print(grid.draw(Logo_E2020), vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
    print(p1, vp = viewport(layout.pos.row = icol, layout.pos.col = 2)) 
    grid.text(MainTitle, vp = viewport(layout.pos.row = 1, layout.pos.col = 2))
    
  } else {
    pushViewport(viewport(layout = grid.layout(nrow=2, Heights = unit(c(1,4), "null"), ncol=3,widths = unit(c(1,4,1), "null"), heigths = unit(c(.1,1),"null"))))
    print(grid.draw(Logo_E2020), vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
    print(p1, vp = viewport(layout.pos.row = icol, layout.pos.col = 2)) 
    grid.text(MainTitle, vp = viewport(layout.pos.row = 1, layout.pos.col = 2))
  }
}


  dev.off() # Close the file
  
  
